package birt.test;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Level;

import javax.servlet.ServletContext;

import org.eclipse.birt.core.exception.BirtException;
import org.eclipse.birt.core.framework.IPlatformContext;
import org.eclipse.birt.core.framework.Platform;
import org.eclipse.birt.core.framework.PlatformServletContext;
import org.eclipse.birt.report.engine.api.EngineConfig;
import org.eclipse.birt.report.engine.api.EngineConstants;
import org.eclipse.birt.report.engine.api.IReportEngine;
import org.eclipse.birt.report.engine.api.IReportEngineFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class BirtEngine {
	
	private static final Logger logger = LoggerFactory.getLogger(BirtEngine.class);
	private static IReportEngine engine;
	private static Level level = Level.OFF;
	private static String logDirectory = null;
	
	public static IReportEngine getBirtEngine() {
        return engine;
    }
	
	public static synchronized void destroyBirtEngine() {
		if (engine != null) {
			engine.destroy();
		}
		Platform.shutdown();
	}
	
	public Object clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException();
    }
	
	public static synchronized void initBirtEngine(ServletContext servletContext) {
		if (engine == null) {
			loadEngineProperties();
			
			IPlatformContext context = new PlatformServletContext(servletContext);
			EngineConfig config = new EngineConfig();
			@SuppressWarnings("unchecked")
			HashMap<String, Object> appContext = config.getAppContext();
			appContext.put(EngineConstants.APPCONTEXT_CLASSLOADER_KEY, BirtEngine.class.getClassLoader());
			config.setLogConfig(logDirectory, level);
			config.setEngineHome("");
			config.setPlatformContext(context);
//			config.setResourcePath(BirtEngine.class.getResource("/").getPath());
			config.setResourcePath(servletContext.getRealPath("/"));
			config.setAppContext(appContext);
			logger.info("Engine resource path: " + config.getResourcePath());
			
			try {
				Platform.startup(config);
			} catch (BirtException e) {
				logger.error(e.getMessage(), e);
			}
			
			IReportEngineFactory factory = (IReportEngineFactory)Platform.createFactoryObject(IReportEngineFactory.EXTENSION_REPORT_ENGINE_FACTORY);
			engine = factory.createReportEngine(config);
			engine.changeLogLevel(Level.WARNING);
		}
	}
	
	private static void loadEngineProperties() {
		try {
			ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
			Properties engineProperties = new Properties();
			InputStream input = classLoader.getResourceAsStream("BirtConfig.properties");
			engineProperties.load(input);
			input.close();
			
			String logLevel = engineProperties.getProperty("logLevel");
			if ("SEVERE".equalsIgnoreCase(logLevel)) {
                level = Level.SEVERE;
            } else if ("WARNING".equalsIgnoreCase(logLevel)) {
                level = Level.WARNING;
            } else if ("INFO".equalsIgnoreCase(logLevel)) {
                level = Level.INFO;
            } else if ("CONFIG".equalsIgnoreCase(logLevel)) {
                level = Level.CONFIG;
            } else if ("FINE".equalsIgnoreCase(logLevel)) {
                level = Level.FINE;
            } else if ("FINER".equalsIgnoreCase(logLevel)) {
                level = Level.FINER;
            } else if ("FINEST".equalsIgnoreCase(logLevel)) {
                level = Level.FINEST;
            } else if ("ALL".equalsIgnoreCase(logLevel)) {
                level = Level.ALL;
            }
			logDirectory = engineProperties.getProperty("logDirectory");
			logger.info("Log Directory: " + logDirectory);
			logger.info("Log Level: " + level);
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
	}

}
