package birt.test.action;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import net.sourceforge.stripes.action.DefaultHandler;
import net.sourceforge.stripes.action.ForwardResolution;
import net.sourceforge.stripes.action.Resolution;
import net.sourceforge.stripes.action.StreamingResolution;

import org.eclipse.birt.report.engine.api.EngineException;
import org.eclipse.birt.report.engine.api.HTMLRenderOption;
import org.eclipse.birt.report.engine.api.HTMLServerImageHandler;
import org.eclipse.birt.report.engine.api.IHTMLRenderOption;
import org.eclipse.birt.report.engine.api.IRenderOption;
import org.eclipse.birt.report.engine.api.IReportEngine;
import org.eclipse.birt.report.engine.api.IReportRunnable;
import org.eclipse.birt.report.engine.api.IRunAndRenderTask;
import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.activity.SemanticException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import birt.test.BirtEngine;
import birt.test.data.SampleTableEntry;

public class DetailsActionBean extends BaseActionBean {

	private static final Logger logger = LoggerFactory.getLogger(DetailsActionBean.class);
	private static final String VIEW = "/WEB-INF/details.jsp";
	private static final String NEXT = "/WEB-INF/embedded.jsp";
	private static final String DIRECT = "/WEB-INF/direct_input.jsp";
	
	private String page = "";
	private String content = "";
	
	public String getContent() {
		return content;
	}

	public void setContent(String content) {
//		final String TABLE = "</table>";
		String html = content;
//		int start = html.indexOf("<table");
//		int end = html.lastIndexOf(TABLE);
//		html = html.substring(start, end + TABLE.length());
		this.content = html;
//		this.content = content;
	}

	public String getPage() {
		return page;
	}

	public void setPage(String page) {
		logger.info("Page Location: " + page);
		this.page = page;
	}

	@DefaultHandler
	public Resolution view() throws EngineException, SemanticException, IOException {
		logger.info("Generating table report using report event");
		
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		HTMLRenderOption options = setHTMLOptions(output, false);
		
		if (generateReport(options)) {
			return new StreamingResolution("text/html", new ByteArrayInputStream(output.toByteArray()));
		} else {
			output.close();
			return new ForwardResolution(VIEW);
		}
	}
	
	public Resolution embedded() throws SemanticException, EngineException, IOException {
		logger.info("Generating embedded report using report event");
		
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		HTMLRenderOption options = setHTMLOptions(output, false);
		
		File f;
		try {
			f = File.createTempFile("report", ".html");
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
			return new ForwardResolution(VIEW);
		}

		setPage(getContext().getServletContext().getContextPath() + "/temp/" + f.getName());
		String tempFile = getContext().getServletContext().getRealPath("/temp/" + f.getName());
		f.delete();
		f = new File(tempFile);
		
		FileOutputStream fs = new FileOutputStream(f);
		
		if (generateReport(options)) {
			fs.write(output.toByteArray());
			fs.close();
			output.close();
			return new ForwardResolution(NEXT);
		} else {
			output.close();
			return new ForwardResolution(VIEW);
		}
	}
	
	public Resolution direct() throws SemanticException, EngineException, IOException {
		logger.info("Generating direct input report using report event");
		
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		HTMLRenderOption options = setHTMLOptions(output, true);
		
		if (generateReport(options)) {
			setContent(new String(output.toByteArray()));
			output.close();
			return new ForwardResolution(DIRECT);
		} else {
			output.close();
			return new ForwardResolution(VIEW);
		}
	}

	private HTMLRenderOption setHTMLOptions(ByteArrayOutputStream output, boolean embed) {
		HTMLRenderOption options = new HTMLRenderOption();
		options.setOutputFormat(IHTMLRenderOption.OUTPUT_FORMAT_HTML);
		options.setOutputStream(output);
		options.setEmbeddable(embed);
		options.setBaseImageURL(getContext().getRequest().getContextPath() + "/images");
		options.setImageDirectory(getContext().getServletContext().getRealPath("/images"));
		options.setImageHandler(new HTMLServerImageHandler());
		options.setMasterPageContent(false);
		logger.info("Is embedded: " + options.getEmbeddable());
		return options;
	}

	private boolean generateReport(IRenderOption options) throws EngineException, SemanticException {
		List<SampleTableEntry> values = new ArrayList<SampleTableEntry>();
		for (int i = 1; i < 21; i++) {
			values.add(new SampleTableEntry("Entry " + i, (int)(Math.random() * 100), Math.random() * 100.0, "This is a comment for entry " + i));
		}
		
		IReportEngine engine = BirtEngine.getBirtEngine();
		if (engine == null) {
			logger.error("BIRT Engine is not initialized!");
			return false;
		}
		
		IReportRunnable design = engine.openReportDesign(getContext()
				.getServletContext().getRealPath("/reports") + "/details.rptdesign");
		IRunAndRenderTask task = engine.createRunAndRenderTask(design);
		
		@SuppressWarnings("unchecked")
		Map<String, Object> context = task.getAppContext();
		DesignElementHandle handle = design.getDesignHandle()
				.getModuleHandle().findElement("detailsTable");
		context.put("values", values);
		handle.setProperty("dataSet", "Scripted Data Set");
		
		task.setLocale(getContext().getLocale());
		task.setRenderOption(options);
		task.setAppContext(context);
		
		task.run();
		task.close();
		
		return true;
	}
}
