package birt.test.action;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.birt.report.engine.api.EngineException;
import org.eclipse.birt.report.engine.api.HTMLRenderOption;
import org.eclipse.birt.report.engine.api.HTMLServerImageHandler;
import org.eclipse.birt.report.engine.api.IHTMLRenderOption;
import org.eclipse.birt.report.engine.api.IRenderOption;
import org.eclipse.birt.report.engine.api.IReportEngine;
import org.eclipse.birt.report.engine.api.IReportRunnable;
import org.eclipse.birt.report.engine.api.IRunAndRenderTask;
import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.activity.SemanticException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import birt.test.BirtEngine;
import birt.test.data.SampleEntity;

import net.sourceforge.stripes.action.DefaultHandler;
import net.sourceforge.stripes.action.ForwardResolution;
import net.sourceforge.stripes.action.Resolution;
import net.sourceforge.stripes.action.StreamingResolution;


public class HomeActionBean extends BaseActionBean {
	
	private static final Logger logger = LoggerFactory.getLogger(HomeActionBean.class);
	private static final String VIEW = "/WEB-INF/home.jsp";
	private String title = "Home";
	
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	@DefaultHandler
	public Resolution view() {
		setTitle("Home");
		logger.info("Entering start page in default event");
		return new ForwardResolution(VIEW);
	}
	
	public Resolution report() throws SemanticException, EngineException {
		setTitle("Report");
		logger.info("Generating chart report using report event");
		
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		IHTMLRenderOption options = new HTMLRenderOption();
		options.setOutputFormat(IHTMLRenderOption.OUTPUT_FORMAT_HTML);
		options.setOutputStream(output);
		options.setEmbeddable(true);
		options.setBaseImageURL(getContext().getRequest().getContextPath() + "/images");
		options.setImageDirectory(getContext().getServletContext().getRealPath("/images"));
		options.setImageHandler(new HTMLServerImageHandler());
		options.setMasterPageContent(false);
		
		if (generateReport(options)) {
			return new StreamingResolution("text/html", new ByteArrayInputStream(output.toByteArray()));
		} else {
			return new ForwardResolution(VIEW);
		}
	}
	
	private boolean generateReport(IRenderOption options) throws EngineException, SemanticException {
		List<SampleEntity> entities = new ArrayList<SampleEntity>();
		entities.add(new SampleEntity(new Integer(1), "Item 1"));
		entities.add(new SampleEntity(new Integer(2), "Item 2"));
		entities.add(new SampleEntity(new Integer(2), "Item 3"));
		entities.add(new SampleEntity(new Integer(3), "Item 4"));
		entities.add(new SampleEntity(new Integer(4), "Item 5"));
		entities.add(new SampleEntity(new Integer(4), "Item 6"));
		entities.add(new SampleEntity(new Integer(5), "Item 7"));
		entities.add(new SampleEntity(new Integer(6), "Item 8"));
		
		IReportEngine engine = BirtEngine.getBirtEngine();
		if (engine == null) {
			logger.error("BIRT Engine is not initialized!");
			return false;
		}
		
		IReportRunnable design = engine.openReportDesign(getContext()
				.getServletContext().getRealPath("/reports") + "/report.rptdesign");
		IRunAndRenderTask task = engine.createRunAndRenderTask(design);
		@SuppressWarnings("unchecked")
		Map<String, Object> context = task.getAppContext();
		DesignElementHandle handle = design.getDesignHandle()
				.getModuleHandle().findElement("reportChart");
		context.put("entities", entities);
		handle.setProperty("dataSet", "Scripted Data Set");
		task.setLocale(getContext().getLocale());
		task.setRenderOption(options);
		task.setAppContext(context);
		
		task.run();
		task.close();

		return true;
	}
	
}